/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define local macros folder ;
filename MACROS '..\macros';

* Include macros ;
%include MACROS('*.sas');

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 11: Linear fixed-effects model
*****************************************************************************************************************/

/*****************************************************************************************************************
Subchapter 11.1: Estimation and inference on the overall treatment effect
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 11.1: Fitting a linear fixed-effects model using proc glm- Balanced data
*****************************************************************************************************************/

title "Balanced data";
proc glm
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment Patient Patient*Treatment Patient*Cycle /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	lsmeans Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for least squares means */
		pdiff /* p-values for differences */
		stderr /* standard error */
		tdiff; * t values ;
	estimate 'B-A'
		Treatment			1 -1;
run;
quit;

/*****************************************************************************************************************
Program 11.2: Fitting a linear fixed-effects model using proc glm- Unbalanced data
*****************************************************************************************************************/

title "Unbalanced data";
proc glm
		data=NORMAL.ndata_unbalanced
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment Patient Patient*Treatment Patient*Cycle /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	estimate 'B-A'
		Treatment			1 -1;
run;
quit;

/*****************************************************************************************************************
Program 11.3: Determining the number of observations per treatment
*****************************************************************************************************************/

title "Unbalanced data";
title2 "Number of observations per Treatment";
proc sql;
	select	Treatment,
			count(Y) as nobs
	from NORMAL.ndata_unbalanced
	group by Treatment;
quit;

/*****************************************************************************************************************
Program 11.4: Determining the number of observations per treatment and patient
*****************************************************************************************************************/

title "Unbalanced data";
title2 "Number of observations per Treatment and Patient";
proc sql;
	select	Patient,
			Treatment,
			count(Y) as nobs
	from NORMAL.ndata_unbalanced
	group by Treatment, Patient;
quit;

/*****************************************************************************************************************
Program 11.5: Fitting a linear fixed-effects model using proc glm- Weighted estimate
*****************************************************************************************************************/

title "Unbalanced data";
title2 "Weighted estimate";
proc glm
		data=NORMAL.ndata_unbalanced
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment Patient Patient*Treatment Patient*Cycle /
		alpha=0.05 /* level for CI */
		clparm /* CI for parameter estimates */
		ss1
		ss2
		ss3;
	estimate 'B-A'
		Treatment			 33 -33
		Patient*Treatment	 3  3  3  3  3  3  3  3  3  3  2  1
							-3 -3 -3 -3 -3 -3 -3 -3 -3 -3 -2 -1 /
		divisor=33;
run;
quit;

/*****************************************************************************************************************
Subchapter 11.2: Estimation and inference on individual treatment effects
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 11.6: Estimating individual treatment effects using proc glm  The hard way
*****************************************************************************************************************/

title "Individual treatment effects";
title2 "Balanced data";
proc glm
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment Patient Patient*Treatment Patient*Cycle /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	estimate 'B-A,1' Treatment 1 -1
		Patient*Treatment	1 0 0 0 0 0 0 0 0 0 0 0
							-1 0 0 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,2' Treatment 1 -1
		Patient*Treatment	0 1 0 0 0 0 0 0 0 0 0 0
							0 -1 0 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,3' Treatment 1 -1
		Patient*Treatment	0 0 1 0 0 0 0 0 0 0 0 0
							0 0 -1 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,4' Treatment 1 -1
		Patient*Treatment	0 0 0 1 0 0 0 0 0 0 0 0
							0 0 0 -1 0 0 0 0 0 0 0 0;
	estimate 'B-A,5' Treatment 1 -1
		Patient*Treatment	0 0 0 0 1 0 0 0 0 0 0 0
							0 0 0 0 -1 0 0 0 0 0 0 0;
	estimate 'B-A,6' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 1 0 0 0 0 0 0
							0 0 0 0 0 -1 0 0 0 0 0 0;
	estimate 'B-A,7' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 0 1 0 0 0 0 0
							0 0 0 0 0 0 -1 0 0 0 0 0;
	estimate 'B-A,8' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 0 0 1 0 0 0 0
							0 0 0 0 0 0 0 -1 0 0 0 0;
	estimate 'B-A,9' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 0 0 0 1 0 0 0
							0 0 0 0 0 0 0 0 -1 0 0 0;
	estimate 'B-A,10' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 0 0 0 0 1 0 0
							0 0 0 0 0 0 0 0 0 -1 0 0;
	estimate 'B-A,11' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 0 0 0 0 0 1 0
							0 0 0 0 0 0 0 0 0 0 -1 0;
	estimate 'B-A,12' Treatment 1 -1
		Patient*Treatment	0 0 0 0 0 0 0 0 0 0 0 1
							0 0 0 0 0 0 0 0 0 0 0 -1;
run;
quit;

/*****************************************************************************************************************
Program 11.7: Estimating individual treatment effects using proc glm  The macro way
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_full_global(
	array=vfull_unbalanced,
	subject=Patient,
	treatment=Treatment,
	data=NORMAL.ndata_unbalanced
)

title "Individual treatment effects";
title2 "Unbalanced data";
proc glm
		data=NORMAL.ndata_unbalanced
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment Patient Patient*Treatment Patient*Cycle /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	%estimate_full(array=vfull_unbalanced, random=)
run;
quit;

* Delete global macro variable array ;
%estimate_full_free(array=vfull_unbalanced)

* Log global macro variables ;
%put _GLOBAL_;

title; * clear all titles ;
