/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016
Modified by Artur Araujo
April 2018

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define local macros folder ;
filename MACROS '..\macros';

* Include macros ;
%include MACROS('*.sas');

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 13: Linear mixed-effects model
*****************************************************************************************************************/

/*****************************************************************************************************************
Subchapter 13.1: Estimation and inference on the overall treatment effect
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 13.1: Fitting a linear mixed-effects model using proc mixed  Balanced data
*****************************************************************************************************************/

title "Balanced data";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		covb /* covariance matrix of fixed effects estimates */
		ddfm=KR /* inference based on Kenward and Roger's method */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient Patient*Cycle Patient*Treatment;
	lsmeans Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for least squares means */
		diff; * difference between each pair of LS means ;
run;

/*****************************************************************************************************************
Program 13.2: Fitting a linear mixed-effects model using proc mixed  Unbalanced data
*****************************************************************************************************************/

title "Unbalanced data";
proc mixed
		data=NORMAL.ndata_unbalanced
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		covb /* covariance matrix of fixed effects estimates */
		ddfm=KR /* inference based on Kenward and Roger's method */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient Patient*Cycle Patient*Treatment;
	lsmeans Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for least squares means */
		diff; * difference between each pair of LS means ;
run;

/*****************************************************************************************************************
Subchapter 13.2: Estimation and inference on individual treatment effects
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 13.3: Estimating individual treatment effects using 'proc mixed'  Kenward and Rogers inference
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Balanced data";
title3 "Kenward-Roger";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		ddfm=KR /* inference based on Kenward and Roger's method */
		htype=2; * type 2 hypothesis test ;
	random Patient Patient*Cycle Patient*Treatment;
	estimate 'B-A,1' Treatment 1 -1 |
		Patient*Treatment	1 0 0 0 0 0 0 0 0 0 0 0
							-1 0 0 0 0 0 0 0 0 0 0 0 /
								alpha=0.05 /* level for CI */
								cl; * t-type CI for estimates ;
	estimate 'B-A,2' Treatment 1 -1 |
		Patient*Treatment 	0 1 0 0 0 0 0 0 0 0 0 0
							0 -1 0 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,3' Treatment 1 -1 |
		Patient*Treatment	0 0 1 0 0 0 0 0 0 0 0 0
							0 0 -1 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,4' Treatment 1 -1 |
		Patient*Treatment	0 0 0 1 0 0 0 0 0 0 0 0
							0 0 0 -1 0 0 0 0 0 0 0 0;
	estimate 'B-A,5' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 1 0 0 0 0 0 0 0
							0 0 0 0 -1 0 0 0 0 0 0 0;
	estimate 'B-A,6' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 1 0 0 0 0 0 0
							0 0 0 0 0 -1 0 0 0 0 0 0;
	estimate 'B-A,7' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 1 0 0 0 0 0
							0 0 0 0 0 0 -1 0 0 0 0 0;
	estimate 'B-A,8' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 1 0 0 0 0
							0 0 0 0 0 0 0 -1 0 0 0 0;
	estimate 'B-A,9' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 1 0 0 0
							0 0 0 0 0 0 0 0 -1 0 0 0;
	estimate 'B-A,10' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 0 1 0 0
							0 0 0 0 0 0 0 0 0 -1 0 0;
	estimate 'B-A,11' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 0 0 1 0
							0 0 0 0 0 0 0 0 0 0 -1 0;
	estimate 'B-A,12' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 0 0 0 1
							0 0 0 0 0 0 0 0 0 0 0 -1;
	ods output estimates=NORMAL.pdata(keep=Label Estimate);
run;

/*****************************************************************************************************************
Program 13.4: Estimating individual treatment effects using proc mixed  Asymptotic inference
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Balanced data";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		htype=2; * type 2 hypothesis test ;
	random Patient Patient*Cycle Patient*Treatment;
	estimate 'B-A,1' Treatment 1 -1 |
		Patient*Treatment	1 0 0 0 0 0 0 0 0 0 0 0
							-1 0 0 0 0 0 0 0 0 0 0 0 /
								alpha=0.05 /* level for CI */
								cl; * t-type CI for estimates ;
	estimate 'B-A,2' Treatment 1 -1 |
		Patient*Treatment 	0 1 0 0 0 0 0 0 0 0 0 0
							0 -1 0 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,3' Treatment 1 -1 |
		Patient*Treatment	0 0 1 0 0 0 0 0 0 0 0 0
							0 0 -1 0 0 0 0 0 0 0 0 0;
	estimate 'B-A,4' Treatment 1 -1 |
		Patient*Treatment	0 0 0 1 0 0 0 0 0 0 0 0
							0 0 0 -1 0 0 0 0 0 0 0 0;
	estimate 'B-A,5' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 1 0 0 0 0 0 0 0
							0 0 0 0 -1 0 0 0 0 0 0 0;
	estimate 'B-A,6' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 1 0 0 0 0 0 0
							0 0 0 0 0 -1 0 0 0 0 0 0;
	estimate 'B-A,7' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 1 0 0 0 0 0
							0 0 0 0 0 0 -1 0 0 0 0 0;
	estimate 'B-A,8' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 1 0 0 0 0
							0 0 0 0 0 0 0 -1 0 0 0 0;
	estimate 'B-A,9' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 1 0 0 0
							0 0 0 0 0 0 0 0 -1 0 0 0;
	estimate 'B-A,10' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 0 1 0 0
							0 0 0 0 0 0 0 0 0 -1 0 0;
	estimate 'B-A,11' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 0 0 1 0
							0 0 0 0 0 0 0 0 0 0 -1 0;
	estimate 'B-A,12' Treatment 1 -1 |
		Patient*Treatment	0 0 0 0 0 0 0 0 0 0 0 1
							0 0 0 0 0 0 0 0 0 0 0 -1;
run;

/*****************************************************************************************************************
Program 13.5: Computing the extreme values of shrunk individual treatment effects
*****************************************************************************************************************/

title "Extreme values of shrunk individual treatment effects";
proc sql;
	select	substring(Label from 5) as Patient,
			Estimate as Extreme
	from NORMAL.pdata
	having Estimate eq min(Estimate)
	or Estimate eq max(Estimate);
quit;

/*****************************************************************************************************************
Program 13.6: Computing summary statistics of shrunk individual treatment effects
*****************************************************************************************************************/

title "Summary statistics of shrunk individual treatment effects";
proc summary
		data=NORMAL.pdata
		print
		maxdec=2
		min q1 mean median q3 max;
	var Estimate;
run;

/*****************************************************************************************************************
Program 13.7: Writing a global variable macro array
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_full_global(
	array=vfull_unbalanced,
	subject=Patient,
	treatment=Treatment,
	data=NORMAL.ndata_unbalanced
)

/*****************************************************************************************************************
Program 13.8: Estimating individual treatment effects using 'proc mixed' and KR inference - The macro way
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Unbalanced data";
title3 "Kenward-Roger";
proc mixed
		data=NORMAL.ndata_unbalanced
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		ddfm=KR /* inference based on Kenward and Roger's method */
		htype=2; * type 2 hypothesis test ;
	random Patient Patient*Cycle Patient*Treatment;
	%estimate_full(array=vfull_unbalanced, options=alpha=0.05 cl)
run;

/*****************************************************************************************************************
Program 13.9: Estimating individual treatment effects using proc mixed and asymptotic inference  The macro way
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Unbalanced data";
proc mixed
		data=NORMAL.ndata_unbalanced
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		htype=2; * type 2 hypothesis test ;
	random Patient Patient*Cycle Patient*Treatment;
	%estimate_full(array=vfull_unbalanced, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_full_free(array=vfull_unbalanced)

/*****************************************************************************************************************
Program 13.10: Estimating individual treatment effects using 'proc mixed'- The alternative way
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Balanced data";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random intercept Treatment Cycle /
		subject=Patient;
	estimate 'B-A,1' Treatment 1 -1 | Treatment 1 -1 /
						subject 1
						alpha=0.05 /* level for CI */
						cl; * t-type CI for estimates ;
	estimate 'B-A,2' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 1;
	estimate 'B-A,3' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 1;
	estimate 'B-A,4' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 1;
	estimate 'B-A,5' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 1;
	estimate 'B-A,6' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 1;
	estimate 'B-A,7' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 0 1;
	estimate 'B-A,8' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 0 0 1;
	estimate 'B-A,9' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 0 0 0 1;
	estimate 'B-A,10' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 0 0 0 0 1;
	estimate 'B-A,11' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 0 0 0 0 0 1;
	estimate 'B-A,12' Treatment 1 -1 | Treatment 1 -1 /
						subject 0 0 0 0 0 0 0 0 0 0 0 1;
run;

/*****************************************************************************************************************
Program 13.11: Estimating individual treatment effects using 'proc mixed'- The alternative macro way
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_full_random_global(
	array=vfull_ndata,
	subject=Patient,
	treatment=Treatment,
	data=NORMAL.ndata,
	ref=first
)

title "Shrunk individual treatment effects";
title2 "Balanced data";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random intercept Treatment Cycle /
		subject=Patient;
	%estimate_full_random(array=vfull_ndata, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_full_random_free(array=vfull_ndata)

* Log global macro variables ;
%put _GLOBAL_;

title; * clear all titles ;
