/*****************************************************************************************************************
Written by Artur Araujo
December 2017

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define local macros folder ;
filename MACROS '..\macros';

* Include macros ;
%include MACROS('*.sas');

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 15: Modelling heterogeneous variances
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 15.1: Fitting a linear mixed-effects model using 'proc mixed' and saving influence statistics
*****************************************************************************************************************/

ods exclude all;
proc mixed data=NORMAL.ndata
		method=REML;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		influence;
	random Patient Patient*Cycle Patient*Treatment;
	ods output influence=WORK.influence;
run;
ods exclude none;

/*****************************************************************************************************************
Program 15.2: Obtaining a dataset with the residual errors and grouping variables
*****************************************************************************************************************/

data WORK.ndata;
	set NORMAL.ndata;
	index=_n_;
run;

data WORK.residual;
	merge
		WORK.ndata
		WORK.influence;
	by index;
	keep Treatment Cycle Residual;
run;

proc datasets lib=WORK nolist;
	delete influence ndata;
quit;

/*****************************************************************************************************************
Program 15.3: Creating boxplots of residual error for each treatment
*****************************************************************************************************************/

* Set graph size in pixels ;
ods graphics on /
	border=off /* suppress graphics border */
	width=720px
	height=720px;

proc sgpanel
		data=WORK.residual
		description="Boxplot of residual error vs treatment"
		noautolegend;
	panelby Treatment;
	vbox Residual /
		group=Treatment
		nofill
		percentile=1;
	rowaxis label="Residual error";
run;

ods graphics off;

proc datasets lib=WORK nolist;
	delete residual;
quit;

/*****************************************************************************************************************
Program 15.4: Fitting a heteroscedastic linear mixed-effects model using proc mixed
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_full_global(
	array=vfull_balanced,
	subject=Patient,
	treatment=Treatment,
	data=NORMAL.ndata
)

title "Heterocedastic linear mixed-effects model";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Cycle;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		covb /* covariance matrix of fixed effects estimates */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient Patient*Cycle Patient*Treatment;
	repeated /
		group=Treatment
		type=vc;
	%estimate_full(array=vfull_balanced, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_full_free(array=vfull_balanced)

* Log global macro variables ;
%put _GLOBAL_;

title; * clear all titles ;
